#![no_std]
#![no_main]
#![feature(type_alias_impl_trait)]
#![feature(never_type)]

mod mouse_hid;
mod mouse_sensor;

use embassy_executor::Spawner;
use embassy_futures::join::join;

use usbd_hid::descriptor::MouseReport;

use crate::{
    mouse_hid::mouse_config::MouseConfig,
    mouse_hid::MouseHID,
    mouse_sensor::{MouseSensor, MouseSensorPins},
};

use {defmt_rtt as _, panic_probe as _};

#[embassy_executor::main]
async fn main(_spawner: Spawner) {
    let p = embassy_rp::init(Default::default());

    let mut mouse_hid = MouseHID::new(p.USB, MouseConfig::new());
    let mut mouse_sensor = MouseSensor::new(
        p.PIN_25,
        MouseSensorPins::new(p.SPI0, p.PIN_2, p.PIN_3, p.PIN_4, p.PIN_5),
        |_| (),
    )
    .await;

    let usb_future = MouseHID::run_usb().await;

    // Do stuff with the class!
    let hid_future = async {
        loop {
            if let Some((x, y)) = mouse_sensor.read_movement().await {
                let report = MouseReport {
                    buttons: 0,
                    x,
                    y,
                    wheel: 0,
                    pan: 0,
                };

                mouse_hid.mouse_report(report).await;
            }
        }
    };

    // Run everything concurrently.
    // If we had made everything `'static` above instead, we could do this using separate tasks instead.
    join(hid_future, usb_future).await;
}
